"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withShortId = exports.generateShortId = exports.isFIR = exports.isCurrency = exports.isKey = exports.isEmpty = exports.isObject = void 0;
const isObject = (obj) => obj !== null && typeof obj === 'object' && !Array.isArray(obj);
exports.isObject = isObject;
const isEmpty = (obj) => {
    if (!(0, exports.isObject)(obj)) {
        throw new Error('Value is not an object');
    }
    for (const prop in obj) {
        if (prop in obj) {
            return false;
        }
    }
    return true;
};
exports.isEmpty = isEmpty;
const isKey = (item) => {
    return item.name.includes('key');
};
exports.isKey = isKey;
const isCurrency = (item) => {
    return ['RUB', 'USD', 'EUR'].includes(item.shortName);
};
exports.isCurrency = isCurrency;
const isFIR = (item) => {
    var _a;
    if (!((_a = item.quests) === null || _a === void 0 ? void 0 : _a.length)) {
        return false;
    }
    return item.quests.reduce((acc, quest) => acc || quest.fir, false);
};
exports.isFIR = isFIR;
const getDecimal = (num) => {
    const decimal = String(num).split('.').pop();
    return Number(`0.${decimal}`);
};
const isDecimal = (num) => {
    return String(num).includes('.');
};
const generateShortId = (index) => {
    const arr = [
        'a', 'b', 'c', 'd', 'e',
        'f', 'g', 'h', 'i', 'j',
        'k', 'l', 'm', 'n', 'o',
        'p', 'q', 'r', 's', 't',
    ];
    const stack = [];
    let result = [];
    if (index < arr.length) {
        result.push(arr[index]);
    }
    else {
        while (index >= 1) {
            const next = index / arr.length;
            if (isDecimal(next)) {
                stack.push(Number(next.toFixed(2)));
            }
            else {
                stack.push(next);
            }
            index = Math.floor(next);
        }
        result = stack.reverse().map(value => {
            let idx = -1;
            if (isDecimal(value)) {
                if (value < 1) {
                    idx = getDecimal(value) * arr.length - 1;
                }
                else {
                    idx = getDecimal(value) * arr.length;
                }
            }
            else {
                idx = 0;
            }
            return arr[idx];
        });
    }
    return result.join('');
};
exports.generateShortId = generateShortId;
const withShortId = (arr) => {
    return arr.map((item, idx) => (Object.assign(Object.assign({}, item), { shortId: (0, exports.generateShortId)(idx) })));
};
exports.withShortId = withShortId;
