"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findDuplicates = exports.gatherChildren = exports.findNodeInTree = exports.findAllParents = exports.findParent = void 0;
const findParent = (root, childId) => {
    if (!childId) {
        return null;
    }
    const traverse = (node) => {
        if (node.children.map(x => x.id).includes(childId)) {
            return node;
        }
        return node.children.reduce((acc, child) => acc || traverse(child), null);
    };
    return traverse(root);
};
exports.findParent = findParent;
const findAllParents = (root, childId) => {
    if (!childId) {
        return [];
    }
    const traverse = (node) => {
        if (node.children.map(x => x.id).includes(childId)) {
            return [node];
        }
        return node.children.reduce((acc, child) => {
            return acc.concat(traverse(child));
        }, []);
    };
    return traverse(root);
};
exports.findAllParents = findAllParents;
const findNodeInTree = (root, id) => {
    if (!id) {
        return null;
    }
    const traverse = (node) => {
        if (id === node.id) {
            return node;
        }
        return node.children.reduce((acc, child) => acc || traverse(child), null);
    };
    return traverse(root);
};
exports.findNodeInTree = findNodeInTree;
const gatherChildren = (node) => {
    const result = [];
    let arr = [node];
    while (arr.length) {
        const node = arr.pop();
        result.push(node);
        arr = arr.concat(node.children);
    }
    return result;
};
exports.gatherChildren = gatherChildren;
const findDuplicates = (ids) => {
    let arr = [];
    return ids.reduce((acc, id) => {
        if (arr.includes(id)) {
            acc.push(id);
        }
        else {
            arr.push(id);
        }
        return acc;
    }, []);
};
exports.findDuplicates = findDuplicates;
