"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parse = void 0;
const _quests = require('../../data/quests.json');
const helpers_1 = require("../helpers");
const tree_1 = require("../helpers/tree");
const constants_1 = require("../constants");
const questsData = _quests;
const findQuestRequiredLevel = (quest) => {
    if (quest.require.level) {
        return quest.require.level;
    }
    return quest.require.quests.reduce((acc, questId) => {
        const q = questsData.find(x => x.id === questId);
        let level = 1;
        if (q) {
            level = findQuestRequiredLevel(q);
        }
        return level > acc ? level : acc;
    }, 1);
};
const findQuestDepth = (quest, _parsedQuests) => {
    let foundQuest = null;
    Object.keys(_parsedQuests).forEach(trader => {
        const roots = _parsedQuests[trader];
        roots.forEach(root => {
            if (!foundQuest) {
                foundQuest = (0, tree_1.findNodeInTree)(root, quest.id);
            }
        });
    });
    // @ts-ignore TODO: TS is treating this as "never"
    return foundQuest ? foundQuest.depth : 100;
};
const mergeQuests = (item, quest, obj, _parsedQuests) => {
    const weight = findQuestRequiredLevel(quest) * 5;
    const depth = findQuestDepth(quest, _parsedQuests) * 20;
    const newWeight = weight + depth;
    const oldWeight = item.weight || Number.MAX_SAFE_INTEGER;
    const updated = Object.assign(Object.assign({}, item), { weight: oldWeight < newWeight ? oldWeight : newWeight, isKey: item.isKey || obj.type === constants_1.ObjectiveType.Key, isCurrency: (0, helpers_1.isCurrency)(item) });
    if (!updated.quests) {
        updated.quests = [];
    }
    updated.quests.push({
        fir: obj.type === constants_1.ObjectiveType.Find,
        name: quest.name,
        quantity: obj.count || 0,
    });
    return updated;
};
/*
const mergeHideout = (item: DataItem, module: HideoutModule, requirement: ModuleRequirement): DataItem => {
  const targetWeight = module.require.reduce((acc, req) =>
    acc += req.type === 'module' ? req.quantity * 1000 : 0,
    module.level * 1000
  )

  const updated: DataItem = {
    ...item,
    isKey: item.isKey || isKey(item),
    isCurrency: item.isCurrency || isCurrency(item),
    weight: item.weight && item.weight < targetWeight
      ? item.weight
      : targetWeight
  }

  if (!updated.hideout) {
    updated.hideout = []
  }

  updated.hideout.push({
    level: module.level,
    module: module.module,
    quantity: requirement.quantity,
  })

  return updated
}
*/
const parse = (_parsedQuests) => {
    const itemsData = require('../../data/items.json');
    let result = [];
    questsData.forEach(quest => {
        quest.objectives.forEach(obj => {
            if ([
                constants_1.ObjectiveType.Collect,
                constants_1.ObjectiveType.Find,
                constants_1.ObjectiveType.Key,
            ].includes(obj.type)) {
                const targetItemIds = obj.target ? [obj.target] : [];
                targetItemIds.forEach(itemId => {
                    const idx = result.findIndex(x => x.id === itemId);
                    if (idx > -1) {
                        const item = result[idx];
                        const updated = mergeQuests(item, quest, obj, _parsedQuests);
                        result[idx] = updated;
                    }
                    else {
                        const item = itemsData[itemId];
                        const updated = mergeQuests(item, quest, obj, _parsedQuests);
                        result.push(updated);
                    }
                });
            }
        });
    });
    /*
    const hideoutData = require('../../data/hideout.json') as HideoutData
    hideoutData.modules.forEach(module => {
      module.require.forEach(requirement => {
        if (requirement.type === 'item') {
          const itemId = requirement.name
          const idx = result.findIndex(x => x.id === itemId)
  
          if (idx > -1) {
            const item: DataItem = result[idx]
            const updated = mergeHideout(item, module, requirement)
            result[idx] = updated
          } else {
            const item: DataItem = itemsData[itemId]
            const updated = mergeHideout(item, module, requirement)
            result.push(updated)
          }
        }
      })
    })
    */
    return result;
};
exports.parse = parse;
